/*-
 * %%----------------------------------------------------------------------------------------------
 * Solidify Framework - Solidify Frontend - token.dialog.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  ChangeDetectionStrategy,
  Component,
  Inject,
  Injector,
  OnInit,
} from "@angular/core";
import {
  MAT_DIALOG_DATA,
  MatDialogRef,
} from "@angular/material/dialog";
import {OAuth2Service} from "../../../auth/oauth2.service";
import {JwtTokenHelper} from "../../../helpers/jwt-token.helper";
import {Token} from "../../../models/token.model";
import {NotificationService} from "../../../services/notification.service";
import {ClipboardUtil} from "../../../utils/clipboard.util";
import {DateUtil} from "../../../utils/date.util";
import {AbstractInternalDialog} from "../abstract-internal/abstract-internal.dialog";

@Component({
  selector: "solidify-token-dialog",
  templateUrl: "./token.dialog.html",
  styleUrls: ["./token.dialog.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class TokenDialog extends AbstractInternalDialog<TokenDialogData> implements OnInit {
  tokenDecoded: Token;

  expiration: string = "";

  get token(): string {
    return this.oauth2Service.getAccessToken();
  }

  constructor(protected readonly _dialogRef: MatDialogRef<TokenDialog>,
              public readonly oauth2Service: OAuth2Service,
              public readonly notificationService: NotificationService,
              @Inject(MAT_DIALOG_DATA) public readonly data: TokenDialogData,
              protected readonly _injector: Injector) {
    super(_injector, _dialogRef, data);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.tokenDecoded = JwtTokenHelper.decodeToken(this.token);
    this.expiration = DateUtil.convertDateToDateTimeString(new Date(this.tokenDecoded.exp * 1000));
  }

  copy(): void {
    if (ClipboardUtil.copyStringToClipboard(this.token)) {
      this.notificationService.showInformation(this.labelTranslateInterface.tokenDialogTokenCopiedToClipboardSuccess);
    } else {
      this.notificationService.showWarning(this.labelTranslateInterface.tokenDialogTokenCopiedToClipboardFail);
    }
  }
}

export interface TokenDialogData {
  extraInformation: TokenDialogKeyValue[];
}

export interface TokenDialogKeyValue {
  key: string;
  value: (token: Token | any) => string;
}
