/*-
 * %%----------------------------------------------------------------------------------------------
 * Solidify Framework - Solidify Frontend - citations.presentational.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  ChangeDetectionStrategy,
  Component,
  Inject,
  Input,
  OnInit,
} from "@angular/core";
import {FormControl} from "@angular/forms";
import {tap} from "rxjs/operators";
import {LABEL_TRANSLATE} from "../../../injection-tokens/label-to-translate.injection-token";
import {Citation} from "../../../models/dto/citation.model";
import {LabelTranslateInterface} from "../../../models/label-translate.model";
import {NotificationService} from "../../../services/notification.service";
import {isNonEmptyArray} from "../../../tools/is/is.tool";
import {ClipboardUtil} from "../../../utils/clipboard.util";
import {AbstractPresentational} from "../abstract/abstract.presentational";

@Component({
  selector: "solidify-citations",
  templateUrl: "./citations.presentational.html",
  styleUrls: ["./citations.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class CitationsPresentational extends AbstractPresentational implements OnInit {
  formControl = new FormControl();

  private _listCitations: Citation[];

  selectedCitation: Citation = undefined;

  @Input()
  set listCitations(value: Citation[]) {
    this._listCitations = value;

    if (isNonEmptyArray(this.listCitations)) {
      this.selectedCitation = this.listCitations[0];
    } else {
      this.selectedCitation = undefined;
    }
    this.formControl.setValue(this.selectedCitation);
  }

  get listCitations(): Citation[] {
    return this._listCitations;
  }

  constructor(private readonly _notificationService: NotificationService,
              @Inject(LABEL_TRANSLATE) readonly labelTranslate: LabelTranslateInterface) {
    super();
  }

  ngOnInit(): void {
    super.ngOnInit();

    this.subscribe(this.formControl.valueChanges.pipe(
      tap(value => {
        this.selectedCitation = value;
      }),
    ));
  }

  copyToClipboard(): void {
    if (ClipboardUtil.copyStringToClipboard(this.selectedCitation.text)) {
      this._notificationService.showInformation(this.labelTranslate.notificationCopiedToClipboard);
    }
  }
}
