/*-
 * %%----------------------------------------------------------------------------------------------
 * Solidify Framework - Solidify Frontend - panel-expandable.presentational.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  animate,
  state,
  style,
  transition,
  trigger,
} from "@angular/animations";
import {
  ChangeDetectionStrategy,
  Component,
  HostBinding,
  Injector,
  Input,
  Output,
} from "@angular/core";
import {
  BehaviorSubject,
  Observable,
} from "rxjs";
import {
  isFalse,
  isTrue,
} from "../../../tools/is/is.tool";
import {ObservableUtil} from "../../../utils/observable.util";
import {AbstractInternalPresentational} from "../abstract-internal/abstract-internal.presentational";

@Component({
  selector: "solidify-panel-expandable",
  templateUrl: "./panel-expandable.presentational.html",
  styleUrls: ["./panel-expandable.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
  animations: [
    trigger("buttonAnimation", [
      state("void", style({transform: "rotate(180deg)"})),
      transition(":enter", animate("600ms ease")),
      transition(":leave", animate("300ms ease")),
    ]),
    trigger("contentAnimation", [
      state("void", style({transform: "translateY(-100%)"})),
      transition(":enter", animate("600ms ease")),
      transition(":leave", animate("300ms ease")),
    ]),
  ],
})
export class PanelExpandablePresentational extends AbstractInternalPresentational {
  @Input()
  titleToTranslate: string;

  @Input()
  suffixToTranslate: string;

  @Input()
  isOpen: boolean = false;

  @HostBinding("class.is-closed")
  get isClosed(): boolean {
    return !this.isOpen;
  }

  @Input()
  @HostBinding("class.is-outline")
  isOutline: boolean = false;

  @Input()
  isRequired: boolean;

  @Input()
  tooltip: string | undefined;

  @Input()
  @HostBinding("class.is-panel-collapsable")
  isPanelCollapsable: boolean = true;

  @Input()
  isTitleClickable: boolean = false;

  private readonly _clickOnTitleBS: BehaviorSubject<void> = new BehaviorSubject<void>(undefined);
  @Output("clickOnTitle")
  readonly clickOnTitle: Observable<void> = ObservableUtil.asObservable(this._clickOnTitleBS);

  constructor(protected readonly _injector: Injector) {
    super(_injector);
  }

  toggle(): void {
    if (isTrue(this.isOutline) || isFalse(this.isPanelCollapsable)) {
      return;
    }
    this.isOpen = !this.isOpen;
  }

  titleClicked($event: MouseEvent): void {
    if (this.isTitleClickable) {
      this._clickOnTitleBS.next();
      $event.stopPropagation();
    }
  }
}
