/*-
 * %%----------------------------------------------------------------------------------------------
 * Solidify Framework - Solidify Frontend - environment.solidify-app.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {Type} from "@angular/core";
import {
  FloatLabelType,
  MatFormFieldAppearance,
} from "@angular/material/form-field";
import {LanguagePartialEnum} from "../enums/partial/language-partial.enum";
import {ThemePartialEnum} from "../enums/partial/theme-partial.enum";
import {KeyValue} from "../models/key-value.model";
import {SolidifyAppUserStateModel} from "../stores/abstract/app-user/app-user-state.model";
import {SolidifyAppUserState} from "../stores/abstract/app-user/app-user.state";
import {SolidifyAppStateModel} from "../stores/abstract/app/app-state.model";
import {SolidifyAppState} from "../stores/abstract/app/app.state";
import {ExtendEnum} from "../types/extend-enum.type";
import {MappingObject} from "../types/mapping-type.type";

export interface DefaultSolidifyAppEnvironment {
  baseHref: string;
  appLanguages: ExtendEnum<LanguagePartialEnum>[];
  defaultLanguage: ExtendEnum<LanguagePartialEnum>;
  appLanguagesTranslate: KeyValue[];
  appThemes: ExtendEnum<ThemePartialEnum>[];
  appThemesTranslate: KeyValue[];
  theme: ExtendEnum<ThemePartialEnum>;
  themeName: string;
  darkMode: boolean;
  highlightJsThemeLight: string;
  highlightJsThemeDark: string;
  reloadAfterLogout: boolean;
  appState: Type<SolidifyAppState<SolidifyAppStateModel>>;
  appUserState: Type<SolidifyAppUserState<SolidifyAppUserStateModel>>;
  routeHomePage: string;
  routeSegmentEdit: string;
  defaultStickyDatatableHeight: number;
  maintenanceMode: boolean;
  appTitle: string;
  appDescription: string;
  elasticApmEnabled: boolean;
  elasticApmUrl: string;
  elasticApmSendErrorsErrors: boolean;
  googleAnalyticsId: string;
  appearanceInputMaterial: MatFormFieldAppearance;
  positionLabelInputMaterial: FloatLabelType;
  displayEmptyRequiredFieldInError: boolean;
  listTagsComponentsWhereClickAllowToEnterInEditMode: string[];
  ribbonEnabled: boolean;
  ribbonColor: string;
  ribbonText: string;
  defaultEnumValuePageSizeOption: number;
  pollingMaxIntervalInSecond: number;
  modalWidth: string;
  actionToDoWhenUpdateAvailable: "do-nothing" | "prompt" | "refresh";
  orcidUrl: string;
  frontendVersionFile: string;
  overlayDelayInMs: number;
  loadBackendModulesInfo: boolean;
  refreshGlobalBannerIntervalInSecond: number;
  labelEnumConverter: ((currentLanguage: ExtendEnum<LanguagePartialEnum>) => string) | undefined;
  displayPrivacyPolicyAndTermsOfUseApprovalDialog: boolean;
  versionPrivacyPolicyAndTermsOfUse: string;
  privacyPolicyLink: MappingObject<ExtendEnum<LanguagePartialEnum>, string>;
  termsOfUseLink: MappingObject<ExtendEnum<LanguagePartialEnum>, string>;
}
