/*-
 * %%----------------------------------------------------------------------------------------------
 * Solidify Framework - Solidify Frontend - file-visualizer.helper.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {SOLIDIFY_CONSTANTS} from "../../constants";
import {FileStatusEnum} from "../../enums/upload/file-status.enum";
import {LabelTranslateInterface} from "../../models/label-translate.model";
import {isTruthyObject} from "../../tools/is/is.tool";
import {MARK_AS_TRANSLATABLE} from "../../utils/translate.util";
import {FileInput} from "../models/file-info.model";
import {AbstractFileVisualizer} from "../services/abstract-file-visualizer.service";

export class FileVisualizerHelper {
  static canHandle(fileVisualizers: AbstractFileVisualizer[], fileInfo: FileInput): boolean {
    if (!isTruthyObject(fileInfo)) {
      return false;
    }
    if (fileInfo.dataFile.status !== FileStatusEnum.READY && fileInfo.dataFile.status !== FileStatusEnum.CONFIRMED && fileInfo.dataFile.status !== FileStatusEnum.TO_BE_CONFIRMED) {
      return false;
    }
    return fileVisualizers.some(value => {
      const computeCanHandleSize = value.canHandleWithSize(fileInfo);
      const computeCanHandle = value.canHandle(fileInfo);
      return computeCanHandleSize && computeCanHandle;
    });
  }

  static errorMessage(fileVisualizers: AbstractFileVisualizer[], fileInfo: FileInput, labelTranslateInterface: LabelTranslateInterface): string {
    if (!fileVisualizers.every(value => value.canHandleWithSize(fileInfo) && value.canHandle(fileInfo))) {
      return this.cannotHandleErrorMessage(fileVisualizers[0], fileInfo, labelTranslateInterface);
    }
  }

  static getFileExtension(fileName: string): string {
    return fileName?.split(".").pop();
  }

  static cannotHandleErrorMessage(fileVisualizers: AbstractFileVisualizer, fileInfo: FileInput, labelTranslateInterface: LabelTranslateInterface): string {
    if (!fileVisualizers.canHandleByExtension(fileInfo)) {
      return MARK_AS_TRANSLATABLE(labelTranslateInterface.notificationPreviewFileNotSupported);
    } else if (!fileVisualizers.canHandleByMimeTypeContentType(fileInfo)) {
      return MARK_AS_TRANSLATABLE(labelTranslateInterface.notificationPreviewFileContentTypeNotSupported);
    } else if (!fileVisualizers.canHandleWithSize(fileInfo)) {
      return MARK_AS_TRANSLATABLE(labelTranslateInterface.notificationPreviewFileTooBig);
    }
    return SOLIDIFY_CONSTANTS.STRING_EMPTY;
  }
}
