/*-
 * %%----------------------------------------------------------------------------------------------
 * Solidify Framework - Solidify Frontend - data-table-columns.model.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {DataTableFieldTypeEnum} from "../../enums/datatable/data-table-field-type.enum";
import {OrderEnum} from "../../enums/order.enum";
import {DataTestPartialEnum} from "../../enums/partial/data-test-partial.enum";
import {ResourceNameSpace} from "../../stores/abstract/resource/resource-namespace.model";
import {ResourceState} from "../../stores/abstract/resource/resource.state";
import {ExtendEnum} from "../../types/extend-enum.type";
import {MappingObject} from "../../types/mapping-type.type";
import {BaseResourceWithLabels} from "../dto/base-resource-with-labels.model";
import {BaseResourceType} from "../dto/base-resource.model";
import {KeyValue} from "../key-value.model";
import {Sort} from "../query-parameters/sort.model";
import {DataTableActions} from "./data-table-actions.model";
import {DataTableComponent} from "./data-table-component.model";

export interface DataTableColumns<TResource extends BaseResourceType = any> {
  field: string; // keyof TResource
  header: string;
  tooltip?: (value: string) => string | undefined;
  type?: DataTableFieldTypeEnum;
  order: OrderEnum;
  filterEnum?: KeyValue[] | BaseResourceWithLabels[];
  dataTest?: ExtendEnum<DataTestPartialEnum>;
  isMultipleFilterEnum?: boolean;
  translate?: boolean;
  isSortable?: boolean;
  isFilterable?: boolean;
  searchableSingleSelectSort?: Sort;
  filterableField?: string; // keyof TResource
  sortableField?: string; // keyof TResource
  resourceNameSpace?: ResourceNameSpace;
  resourceLabelKey?: string;
  resourceValueKey?: string;
  resourceLabelCallback?: (value: any) => string;
  resourceExtraLabelSecondLineCallback?: (value: any) => string;
  resourceState?: ResourceState<any, any>;
  useResourceLabelCallback?: boolean | undefined;
  component?: DataTableComponent;
  width?: string | undefined;
  minWidth?: string | undefined;
  maxWidth?: string | undefined;
  alignment?: "left" | "center" | "right";
  breakWord?: boolean | undefined;
  extraSearchQueryParam?: MappingObject<string, string> | undefined;
  isUser?: boolean;
  listCallback?: (value: any) => string;
  maxListItem?: number | undefined;
  conditionallyDisplay?: ((data: TResource) => boolean) | undefined;
  action?: DataTableActions<TResource>;
  centerContent?: boolean;
  isHighlightCell?: ((data: TResource) => boolean) | undefined;
}
