/*-
 * %%----------------------------------------------------------------------------------------------
 * Solidify Framework - Solidify Frontend - abstract-store-dialog.service.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {Inject} from "@angular/core";
import {DeleteDialogData} from "../components/dialogs/delete/delete.dialog";
import {ButtonColorEnum} from "../enums/button-color.enum";
import {StatePartialEnum} from "../enums/partial/state-partial.enum";
import {DefaultSolidifyEnvironment} from "../environments/environment.solidify-defaults";
import {ENVIRONMENT} from "../injection-tokens/environment.injection-token";
import {isNullOrUndefined} from "../tools/is/is.tool";
import {ExtendEnum} from "../types/extend-enum.type";
import {StringUtil} from "../utils/string.util";
import {AbstractBaseService} from "./abstract-base.service";

export abstract class AbstractStoreDialogService extends AbstractBaseService {
  private readonly _messageNotFound: string = "There is no dialog for action '{0}' declared for the state '{1}'. Please add 'if' statement into StoreDialogService class.";

  constructor(@Inject(ENVIRONMENT) private readonly _environment: DefaultSolidifyEnvironment) {
    super();
  }

  protected abstract _deleteDataInternal(state: ExtendEnum<StatePartialEnum>): DeleteDialogData | undefined;

  deleteData(state: ExtendEnum<StatePartialEnum>): DeleteDialogData {
    const deleteDialogData = this._deleteDataInternal(state);
    if (isNullOrUndefined(deleteDialogData)) {
      this._promptError("Delete", state);
    }
    deleteDialogData.colorConfirm = ButtonColorEnum.warn;
    deleteDialogData.colorCancel = ButtonColorEnum.primary;
    return deleteDialogData;
  }

  private _promptError(typeRoute: string, state: ExtendEnum<StatePartialEnum>): void {
    throw Error(StringUtil.format(this._messageNotFound, typeRoute, state));
  }
}
