/*-
 * %%----------------------------------------------------------------------------------------------
 * Solidify Framework - Solidify Frontend - error.service.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  HttpErrorResponse,
  HttpStatusCode,
} from "@angular/common/http";
import {
  Inject,
  Injectable,
} from "@angular/core";
import {TranslateService} from "@ngx-translate/core";
import {DefaultSolidifyEnvironment} from "../environments/environment.solidify-defaults";
import {ErrorHelper} from "../helpers/error.helper";
import {ENVIRONMENT} from "../injection-tokens/environment.injection-token";
import {LABEL_TRANSLATE} from "../injection-tokens/label-to-translate.injection-token";
import {LabelTranslateInterface} from "../models/label-translate.model";
import {isNullOrUndefined} from "../tools/is/is.tool";
import {AbstractBaseService} from "./abstract-base.service";

@Injectable({
  providedIn: "root",
})
export class ErrorService extends AbstractBaseService {
  constructor(private readonly _translate: TranslateService,
              @Inject(ENVIRONMENT) private readonly _environment: DefaultSolidifyEnvironment,
              @Inject(LABEL_TRANSLATE) readonly labelTranslate: LabelTranslateInterface) {
    super();
  }

  getClientMessage(error: Error): string {
    if (!navigator.onLine) {
      return this._translate.instant(this.labelTranslate.notificationHttpOfflineToTranslate);
    }
    return error.message ? error.message : error.toString();
  }

  getClientStack(error: Error): string {
    return error.stack;
  }

  getServerMessage(error: HttpErrorResponse): string {
    if (error.status === HttpStatusCode.Forbidden) {
      return this._translate.instant(this.labelTranslate.notificationHttpForbiddenToTranslate);
    }
    if (error.status === HttpStatusCode.Unauthorized) {
      if (isNullOrUndefined(error.error) || error.error.error === ErrorHelper.INVALID_TOKEN_ERROR) {
        return this._translate.instant(this.labelTranslate.notificationHttpInvalidTokenToTranslate);
      } else {
        return this._translate.instant(this.labelTranslate.notificationHttpUnauthorizedToTranslate);
      }
    }
    if (error.status === HttpStatusCode.BadRequest) {
      return this._translate.instant(this.labelTranslate.notificationHttpBadRequestToTranslate);
    }
    if (error.status === HttpStatusCode.NotFound) {
      return this._translate.instant(this.labelTranslate.notificationHttpNotFoundToTranslate);
    }
    if (error.status === HttpStatusCode.InternalServerError) {
      return this._translate.instant(this.labelTranslate.notificationHttpInternalErrorToTranslate);
    }
    return error.message;
  }

  getServerStack(error: HttpErrorResponse): string {
    return error.message;
  }
}
