/*-
 * %%----------------------------------------------------------------------------------------------
 * Solidify Framework - Solidify Frontend - solidify-frontend.module.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {CommonModule} from "@angular/common";
import {
  Inject,
  NgModule,
} from "@angular/core";
import {
  FormsModule,
  ReactiveFormsModule,
} from "@angular/forms";
import {
  ApmModule,
  ApmService,
} from "@elastic/apm-rum-angular";
import {FontAwesomeModule} from "@fortawesome/angular-fontawesome";
import {TranslateModule} from "@ngx-translate/core";
import {NgxsModule} from "@ngxs/store";
import {
  PdfViewerComponent,
  PdfViewerModule,
} from "ng2-pdf-viewer";
import {CookieModule} from "ngx-cookie";
import {QuillModule} from "ngx-quill";
import {AdditionalInformationPanelContainer} from "./components/containers/additional-information-panel/additional-information-panel.container";
import {EmptyContainer} from "./components/containers/empty-container/empty.container";
import {MarkdownViewerContainer} from "./components/containers/markdown-viewer/markdown-viewer.container";
import {TabsContainer} from "./components/containers/tabs/tabs.container";
import {BaseActionDialog} from "./components/dialogs/base-action/base-action.dialog";
import {BaseInfoDialog} from "./components/dialogs/base-info/base-info.dialog";
import {ConfirmDialog} from "./components/dialogs/confirm/confirm.dialog";
import {DataTableLooseSelectionDialog} from "./components/dialogs/data-table-loose-selection/data-table-loose-selection.dialog";
import {DeleteDialog} from "./components/dialogs/delete/delete.dialog";
import {FirstLoginDialog} from "./components/dialogs/first-login/first-login.dialog";
import {KeywordPasteDialog} from "./components/dialogs/keyword-paste/keyword-paste.dialog";
import {PrivacyPolicyTermsOfUseApprovalDialog} from "./components/dialogs/privacy-policy-terms-of-use-approval/privacy-policy-terms-of-use-approval.dialog";
import {StatusHistoryDialog} from "./components/dialogs/status-history/status-history.dialog";
import {TokenDialog} from "./components/dialogs/token/token.dialog";
import {UpdateVersionDialog} from "./components/dialogs/update-version/update-version.dialog";
import {BannerPresentational} from "./components/presentationals/banner/banner.presentational";
import {BreadcrumbPresentational} from "./components/presentationals/breadcrumb/breadcrumb.presentational";
import {ButtonIdPresentational} from "./components/presentationals/button-id/button-id.presentational";
import {ButtonOrcidPresentational} from "./components/presentationals/button-orcid/button-orcid.presentational";
import {ButtonRefreshPresentational} from "./components/presentationals/button-refresh/button-refresh.presentational";
import {ButtonStatusHistoryPresentational} from "./components/presentationals/button-status-history/button-status-history.presentational";
import {ButtonToolbarDetailPresentational} from "./components/presentationals/button-toolbar-detail/button-toolbar-detail.presentational";
import {ButtonToolbarListPresentational} from "./components/presentationals/button-toolbar-list/button-toolbar-list.presentational";
import {CitationsPresentational} from "./components/presentationals/citations/citations.presentational";
import {DataTablePresentational} from "./components/presentationals/data-table/data-table.presentational";
import {DetailPresentationPresentational} from "./components/presentationals/detail-presentation/detail-presentation.presentational";
import {EmptyFormControlWrapperPresentational} from "./components/presentationals/empty-form-control-wrapper/empty-form-control-wrapper.presentational";
import {FormErrorWrapperPresentational} from "./components/presentationals/form-error-wrapper/form-error-wrapper.presentational";
import {GlobalBannerPresentational} from "./components/presentationals/global-banner/global-banner.presentational";
import {IconPresentational} from "./components/presentationals/icon/icon.presentational";
import {KeywordInputPresentational} from "./components/presentationals/keyword-input/keyword-input.presentational";
import {LabelPresentational} from "./components/presentationals/label/label.presentational";
import {LanguageSelectorMenuPresentational} from "./components/presentationals/language-selector-menu/language-selector-menu.presentational";
import {LanguageSelectorPresentational} from "./components/presentationals/language-selector/language-selector.presentational";
import {MultiSelectContentPresentational} from "./components/presentationals/multi-select-content/multi-select-content.presentational";
import {MultiSelectDefaultValueContentPresentational} from "./components/presentationals/multi-select-default-value-content/multi-select-default-value-content.presentational";
import {MultiSelectPresentational} from "./components/presentationals/multi-select/multi-select.presentational";
import {PaginatorPresentational} from "./components/presentationals/paginator/paginator.presentational";
import {PanelExpandablePresentational} from "./components/presentationals/panel-expandable/panel-expandable.presentational";
import {RibbonPresentational} from "./components/presentationals/ribbon/ribbon.presentational";
import {RichTextEditorPresentational} from "./components/presentationals/rich-text-editor/rich-text-editor.presentational";
import {SearchInputPresentational} from "./components/presentationals/search-input/search-input.presentational";
import {SearchableMultiSelectContentPresentational} from "./components/presentationals/searchable-multi-select-content/searchable-multi-select-content.presentational";
import {SearchableMultiSelectPresentational} from "./components/presentationals/searchable-multi-select/searchable-multi-select.presentational";
import {SearchableSingleSelectContentPresentational} from "./components/presentationals/searchable-single-select-content/searchable-single-select-content.presentational";
import {SearchableSingleSelectPresentational} from "./components/presentationals/searchable-single-select/searchable-single-select.presentational";
import {SearchableTreeMultiSelectContentPresentational} from "./components/presentationals/searchable-tree-multi-select-content/searchable-tree-multi-select-content.presentational";
import {SearchableTreeMultiSelectPresentational} from "./components/presentationals/searchable-tree-multi-select/searchable-tree-multi-select.presentational";
import {SearchableTreeSingleSelectContentPresentational} from "./components/presentationals/searchable-tree-single-select-content/searchable-tree-single-select-content.presentational";
import {SearchableTreeSingleSelectPresentational} from "./components/presentationals/searchable-tree-single-select/searchable-tree-single-select.presentational";
import {SnackbarPresentational} from "./components/presentationals/snackbar/snackbar.presentational";
import {StarRatingPresentational} from "./components/presentationals/star-rating/star-rating.presentational";
import {StatusPresentational} from "./components/presentationals/status/status.presentational";
import {TableModuleVersionPresentational} from "./components/presentationals/table-module-version/table-module-version.presentational";
import {ThemeSelectorMenuPresentational} from "./components/presentationals/theme-selector-menu/theme-selector-menu.presentational";
import {ThemeSelectorPresentational} from "./components/presentationals/theme-selector/theme-selector.presentational";
import {TourStepPresentational} from "./components/presentationals/tour-step/tour-step.presentational";
import {UrlInputNavigationPresentational} from "./components/presentationals/url-input-navigation/url-input-navigation.presentational";
import {AboutRoutable} from "./components/routables/about/about.routable";
import {ChangelogRoutable} from "./components/routables/changelog/changelog.routable";
import {IconAppSummaryRoutable} from "./components/routables/icon-app-summary/icon-app-summary.routable";
import {MaintenanceModeRoutable} from "./components/routables/maintenance-mode/maintenance-mode.routable";
import {ReleaseNotesRoutable} from "./components/routables/release-notes/release-notes.routable";
import {ServerOfflineModeRoutable} from "./components/routables/server-offline-mode/server-offline-mode.routable";
import {UnableToLoadAppRoutable} from "./components/routables/unable-to-load-app/unable-to-load-app.routable";
import {CookieConsentBannerContainer} from "./cookie-consent/components/containers/cookie-consent-banner/cookie-consent-banner.container";
import {CookieConsentSidebarContainer} from "./cookie-consent/components/containers/cookie-consent-sidebar/cookie-consent-sidebar.container";
import {AutoScrollIntoViewDirective} from "./directives/auto-scroll-into-view/auto-scroll-into-view.directive";
import {AutoFocusDirective} from "./directives/autofocus/auto-focus.directive";
import {FocusFirstElementDirective} from "./directives/autofocus/focus-first-element.directive";
import {AlternativeButtonDirective} from "./directives/button/alternative-button.directive";
import {MatButtonThemeDirective} from "./directives/button/mat-button-theme.directive";
import {RoundButtonDirective} from "./directives/button/round-button.directive";
import {DataTableAnchorDynamicInjectionDirective} from "./directives/data-table-anchor-dynamic-injection/data-table-anchor-dynamic-injection.directive";
import {DataTestDirective} from "./directives/data-test/data-test.directive";
import {FileDropZoneDirective} from "./directives/file-drop-zone/file-drop-zone.directive";
import {FormControlElementRefDirective} from "./directives/form-control-element-ref/form-control-element-ref.directive";
import {OverlayDirective} from "./directives/overlay/overlay.directive";
import {ShortcutDirective} from "./directives/shortcut/shortcut.directive";
import {ButtonSpinnerDirective} from "./directives/spinner/button-spinner.directive";
import {SpinnerDirective} from "./directives/spinner/spinner.directive";
import {TooltipOnEllipsisDirective} from "./directives/tooltip-on-ellipsis/tooltip-on-ellipsis.directive";
import {TooltipDirective} from "./directives/tooltip/tooltip.directive";
import {ValidationDirective} from "./directives/validation/validation.directive";
import {DefaultSolidifyEnvironment} from "./environments/environment.solidify-defaults";
import {FileVisualizerContainer} from "./file-preview/components/containers/file-visualizer.container";
import {FilePreviewDialog} from "./file-preview/components/dialogs/file-preview/file-preview.dialog";
import {ViewerToolBarPresentational} from "./file-preview/components/presentationals/viewer-tool-bar/viewer-tool-bar.presentational";
import {AddExternalComponentDirective} from "./file-preview/directives/add-external-component.directive";
import {FILE_VISUALIZERS} from "./file-preview/services/abstract-file-visualizer.service";
import {DocxPreviewVisualizerService} from "./file-preview/services/docx-preview-file-visualizer.service";
import {ImageFileVisualizerService} from "./file-preview/services/image-file-visualizer.service";
import {MolFileVisualizerService} from "./file-preview/services/mol-file-visualizer.service";
import {PdfFileVisualizerWithToolBarService} from "./file-preview/services/pdf-file-visualizer-with-tool-bar.service";
import {SmeFileVisualizerService} from "./file-preview/services/sme-file-visualizer.service";
import {SoundFileVisualizerService} from "./file-preview/services/sound-file-visualizer.service";
import {TextFileVisualizerService} from "./file-preview/services/text-file-visualizer.service";
import {TiffFileVisualizerService} from "./file-preview/services/tiff-file-visualizer.service";
import {VideoClassicalFileVisualizerService} from "./file-preview/services/video-classical-file-visualizer.service";
import {VisualizationState} from "./file-preview/stores/visualization.state";
import {ENVIRONMENT} from "./injection-tokens/environment.injection-token";
import {MaterialModule} from "./material.module";
import {pipes as otherPipes} from "./pipes/pipes.model";
import {isPipes} from "./pipes/shared-is-pipes/is-pipes.model";
import {
  isNotNullNorUndefinedNorWhiteString,
  isTrue,
} from "./tools/is/is.tool";

// Allow to fix ERROR: node_modules/ngx-tour-core/lib/tour.module.d.ts:4:23 - error TS2314: Generic type 'ModuleWithProviders<T>' requires 1 type argument(s) static forRoot(): ModuleWithProviders;
// declare module "@angular/core" {
//   interface ModuleWithProviders<T = any> {
//     ngModule: Type<T>;
//     providers?: Provider[];
//   }
// }

const routables = [
  AboutRoutable,
  ReleaseNotesRoutable,
  ChangelogRoutable,
  IconAppSummaryRoutable,
  MaintenanceModeRoutable,
  ServerOfflineModeRoutable,
  UnableToLoadAppRoutable,
];
const containers = [
  AdditionalInformationPanelContainer,
  EmptyContainer,
  TabsContainer,
  FileVisualizerContainer,
  CookieConsentSidebarContainer,
  CookieConsentBannerContainer,
  MarkdownViewerContainer,
  GlobalBannerPresentational,
];
const presentationals = [
  BannerPresentational,
  BreadcrumbPresentational,
  ButtonIdPresentational,
  ButtonRefreshPresentational,
  ButtonOrcidPresentational,
  ButtonStatusHistoryPresentational,
  ButtonToolbarDetailPresentational,
  ButtonToolbarListPresentational,
  DataTablePresentational,
  DetailPresentationPresentational,
  IconPresentational,
  EmptyFormControlWrapperPresentational,
  FormErrorWrapperPresentational,
  KeywordInputPresentational,
  LanguageSelectorPresentational,
  LanguageSelectorMenuPresentational,
  MultiSelectPresentational,
  PaginatorPresentational,
  LabelPresentational,
  PanelExpandablePresentational,
  RibbonPresentational,
  RichTextEditorPresentational,
  SearchInputPresentational,
  SearchableMultiSelectPresentational,
  SearchableSingleSelectPresentational,
  SearchableTreeSingleSelectPresentational,
  SearchableTreeMultiSelectPresentational,
  SnackbarPresentational,
  StarRatingPresentational,
  StatusPresentational,
  ThemeSelectorPresentational,
  ThemeSelectorMenuPresentational,
  TourStepPresentational,
  UrlInputNavigationPresentational,
  ViewerToolBarPresentational,
  CitationsPresentational,
  TableModuleVersionPresentational,
];
const contents = [
  SearchableSingleSelectContentPresentational,
  SearchableMultiSelectContentPresentational,
  SearchableTreeSingleSelectContentPresentational,
  SearchableTreeMultiSelectContentPresentational,
  MultiSelectContentPresentational,
  MultiSelectDefaultValueContentPresentational,
];
const dialogs = [
  BaseActionDialog,
  BaseInfoDialog,
  ConfirmDialog,
  DataTableLooseSelectionDialog,
  DeleteDialog,
  FirstLoginDialog,
  StatusHistoryDialog,
  PrivacyPolicyTermsOfUseApprovalDialog,
  TokenDialog,
  UpdateVersionDialog,
  KeywordPasteDialog,
  FilePreviewDialog,
];
const directives = [
  AutoScrollIntoViewDirective,
  AutoFocusDirective,
  FocusFirstElementDirective,
  AlternativeButtonDirective,
  MatButtonThemeDirective,
  RoundButtonDirective,
  DataTestDirective,
  ValidationDirective,
  ShortcutDirective,
  ButtonSpinnerDirective,
  TooltipOnEllipsisDirective,
  TooltipDirective,
  SpinnerDirective,
  FileDropZoneDirective,
  FormControlElementRefDirective,
  DataTableAnchorDynamicInjectionDirective,
  AddExternalComponentDirective,
  OverlayDirective,
];
const pipes = [
  ...isPipes,
  ...otherPipes,
];
const modules = [
  ApmModule,
  PdfViewerModule,
];
const states = [
  VisualizationState,
];

@NgModule({
  declarations: [
    ...routables,
    ...directives,
    ...containers,
    ...presentationals,
    ...contents,
    ...dialogs,
    ...pipes,
  ],
  imports: [
    ...modules,
    CommonModule,
    FontAwesomeModule,
    MaterialModule,
    FormsModule,
    ReactiveFormsModule,
    NgxsModule.forFeature([
      ...states,
    ]),
    TranslateModule.forChild({}),
    CookieModule.forChild(),
    QuillModule.forRoot(),
    // TourMatMenuModule.forRoot(),
  ],
  exports: [
    ...routables,
    ...directives,
    ...containers,
    ...presentationals,
    ...contents,
    ...dialogs,
    ...pipes,
    ...modules,
  ],
  providers: [
    {provide: FILE_VISUALIZERS, useClass: TiffFileVisualizerService, multi: true},
    {provide: FILE_VISUALIZERS, useClass: ImageFileVisualizerService, multi: true},
    {provide: FILE_VISUALIZERS, useClass: SmeFileVisualizerService, multi: true},
    {provide: FILE_VISUALIZERS, useClass: VideoClassicalFileVisualizerService, multi: true},
    {provide: FILE_VISUALIZERS, useClass: PdfFileVisualizerWithToolBarService, multi: true},
    {provide: FILE_VISUALIZERS, useClass: SoundFileVisualizerService, multi: true},
    {provide: FILE_VISUALIZERS, useClass: MolFileVisualizerService, multi: true},
    {provide: FILE_VISUALIZERS, useClass: TextFileVisualizerService, multi: true},
    {provide: FILE_VISUALIZERS, useClass: DocxPreviewVisualizerService, multi: true},
    // TODO use Video VideoFileVisualizerService when all video will be supported and when add a converter
    // {provide: FILE_VISUALIZERS, useClass: VideoFileVisualizerService, multi: true},
    // Visualization of PDF with no toolbar
    // {provide: FILE_VISUALIZERS, useClass: PdfFileVisualizerService, multi: true},
    ApmService,
  ],
  bootstrap: [
    ...dialogs,
    ViewerToolBarPresentational,
    PdfViewerComponent,
  ],
})
export class SolidifyFrontendModule {
  constructor(private readonly _apmService: ApmService,
              @Inject(ENVIRONMENT) protected readonly _environment: DefaultSolidifyEnvironment) {
    if (isTrue(this._environment.elasticApmEnabled) && isNotNullNorUndefinedNorWhiteString(this._environment.elasticApmUrl)) {
      _apmService.init({
        serviceName: this._environment.appTitle,
        serverUrl: this._environment.elasticApmUrl,
        environment: this._environment.production ? "Production" : (isTrue(this._environment.ribbonEnabled) && isNotNullNorUndefinedNorWhiteString(this._environment.ribbonText) ? this._environment.ribbonText : "Development"),
      });
    }
  }
}
